/* Copyright (c) 1997, MpegTV LLC */
#ifndef _pmp_h
#define _pmp_h
#ifndef lint
static const char rcsid_pmp_h[] = "$Header: /net/hopkins/home/tristan/mpeg/dec/RCS/pmp.h,v 1.40 1999/09/25 10:53:30 tristan Exp $";
#endif

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

#include "misc.h"

/*--------------------------------------------------------------------------+
| Macros																	|
+--------------------------------------------------------------------------*/
/* min size of buffer for pmp_print_msg() */
#define MTVP_PMP_PRINT_MSG_BUF_SIZE	(4 * PATH_MAX)

/*--------------------------------------------------------------------------+
| Data types																|
+--------------------------------------------------------------------------*/
/* MTVP_PMP_STRUCT is an opaque type, defined only inside the pmp library */
typedef struct pmp_struct MTVP_PMP_STRUCT;

typedef struct {
	int16		serial_number;
	int16		flags;

/* specify parameter type */
#define		MTVP_PMP_FLAG_SYNCHRONOUS	(1 << 0)	/* used to be MTVP_PMP_FLAG_ACK */

#define		MTVP_PMP_FLAG_TYPE_CUSTOM	(0 << 1)
#define		MTVP_PMP_FLAG_TYPE_VOID		(1 << 1)
#define		MTVP_PMP_FLAG_TYPE_INT		(2 << 1)
#define		MTVP_PMP_FLAG_TYPE_DOUBLE	(3 << 1)
#define		MTVP_PMP_FLAG_TYPE_STRING	(4 << 1)
#define		MTVP_PMP_FLAG_TYPE_RESERVED1	(5 << 1)
#define		MTVP_PMP_FLAG_TYPE_RESERVED2	(6 << 1)
#define		MTVP_PMP_FLAG_TYPE_RESERVED3	(7 << 1)

#define		MTVP_PMP_FLAG_TYPE_MASK		(7 << 1)
#define		MTVP_PMP_FLAG_TYPE(msg)		((msg)->flags & MTVP_PMP_FLAG_TYPE_MASK)

/* note: MTVP_PMP_FLAG_RETURN_STATUS and MTVP_PMP_FLAG_SYNCHRONOUS are exclusives!! */
#define		MTVP_PMP_FLAG_RETURN_STATUS		(1 << 4)	/* request command result */
#define		MTVP_PMP_FLAG_RETURN_ORIGINAL	(1 << 5)	/* copy original message after result */

	u_int16		type;
#if 0 /* Tristan Savatier - Sun Jul 4 00:26:03 PDT 1999 */
//#define		MTVP_PMP_TYPE_ACK	((u_int16)(-1))
#endif /* 0 - Tristan Savatier - Sun Jul 4 00:26:03 PDT 1999 */

	u_int16		size;
} MTVP_PMP_HEADER;

/* standard message data */
typedef union {
	int		data_int;
	double	data_double;
#if 0 /* Tristan Savatier - Sun Jul 4 01:32:37 PDT 1999 */
//	char	data_string[PATH_MAX];
#endif /* 0 - Tristan Savatier - Sun Jul 4 01:32:37 PDT 1999 */
	char	data_string[1];
	char	data_reserved[1024];	/* see note below */ 
} MTVP_PMP_DATA_GENERIC;

/*
 * data_reserved is added only to make sure that MTVP_PMP_DATA_GENERIC is "large enough".
 * The idea is to secretely reserve enough room in unions that contain MTVP_PMP_DATA_GENERIC,
 * to take care of forward compatible extension.
 * For example, this way, old applications that MTVP_PMP_DATA_A2G to read messages will
 * not crash if a newer mtvp sends a longer message (longer than any of the
 * types in th old MTVP_PMP_DATA_A2G union).
 */

/* the following header files may use the types defined above! */
#ifdef WIN32
#include "pmp_win32.h"
#else
#include "pmp_unix.h"
#endif

/*--------------------------------------------------------------------------+
| Macros																	|
+--------------------------------------------------------------------------*/
#define PMP_CREATE_QUEUE_FLAG_ACK	1	// support for synchronous commands (ack)


/*--------------------------------------------------------------------------+
| Externals																	|
+--------------------------------------------------------------------------*/
#ifdef __cplusplus
extern "C" {
#endif

extern int pmp_receive_msg(MTVP_PMP_STRUCT *pmp, MTVP_PMP_HEADER *header, void *data, u_int data_size);
extern int pmp_receive_ack(MTVP_PMP_STRUCT *pmp, MTVP_PMP_HEADER *header, void *data, u_int data_size);

extern int pmp_send_msg(MTVP_PMP_STRUCT *pmp, MTVP_PMP_HEADER *header, const void *data);
extern int pmp_send_ack(MTVP_PMP_STRUCT *pmp, MTVP_PMP_HEADER *header, const void *data);

extern Bool pmp_check(MTVP_PMP_STRUCT *pmp);
#if 0 /* Tristan Savatier - Sat Sep 25 02:23:35 PDT 1999 */
//extern int pmp_init_queue(MTVP_PMP_STRUCT *pmp, u_int queue_id, u_int type_min, u_int type_max);
//extern int pmp_release_queue(MTVP_PMP_STRUCT *pmp);
//extern int pmp_child_init(MTVP_PMP_STRUCT *pmp_child, const MTVP_PMP_STRUCT *pmp_parent);
#endif /* 0 - Tristan Savatier - Sat Sep 25 02:23:35 PDT 1999 */

//extern MTVP_PMP_STRUCT *pmp_create_queue(u_long flags, u_int queue_id, u_int type_min, u_int type_max, u_int type_ack);
extern int pmp_create_queue(MTVP_PMP_STRUCT **result_pmp, u_long flags, u_int queue_id, u_int type_min, u_int type_max, u_int type_ack);
extern void pmp_delete_queue(MTVP_PMP_STRUCT *pmp);
//extern MTVP_PMP_STRUCT *pmp_child_init(const MTVP_PMP_STRUCT *pmp_parent);
extern int pmp_child_init(const MTVP_PMP_STRUCT *pmp_parent, MTVP_PMP_STRUCT **result_pmp_child_ptr);

extern int pmp_parent_init(MTVP_PMP_STRUCT *pmp);
extern const TCHAR * pmp_print_msg(const MTVP_PMP_HEADER *message, const void *data, TCHAR *buffer);

#ifdef __cplusplus
}
#endif

#endif /* _pmp_h */
